#include "swapwindow.hpp"
#include <QVTKWidget.h>
#include <QApplication>
#include <QGridLayout>
#include <QPushButton>

#include <vtkConeSource.h>
#include <vtkPolyDataMapper.h>
#include <vtkRenderWindow.h>
#include <vtkCamera.h>
#include <vtkActor.h>
#include <vtkRenderer.h>
#include <vtkSphereSource.h>
#include <vtkCamera.h>
#include <vtkProperty.h>
#include <vtkLight.h>

int main(int argc, char *argv[])
{
	// QT Stuff
	QApplication app(argc, argv);

#if QT_VERSION >= 0x050000
	QApplication::setStyle("fusion");
#else
	QApplication::setStyle(new QCleanlooksStyle);
#endif

	SwapWindowFrame mySwapWindowFrame;
	mySwapWindowFrame.show();

	return app.exec();
}

SwapWindowFrame::SwapWindowFrame() :
	QWidget()

{
	layout_ = new QGridLayout(this);
	leftPanel_ = new QWidget(this);
	rightPanel_ = new QWidget(this);

	leftLayout_ = new QVBoxLayout(leftPanel_);
	rightLayout_ = new QVBoxLayout(rightPanel_);

	createCone(leftPanel_);
	leftLayout_->addWidget(conewidget_);

	createSpheres(rightPanel_);
	rightLayout_->addWidget(spherewidget_);
	
	layout_->addWidget(leftPanel_, 0, 0);
	layout_->addWidget(rightPanel_, 0, 1);

	QPushButton *swap = new QPushButton("Swap", this);
	connect(swap, SIGNAL(clicked()), this, SLOT(swap()));
	layout_->addWidget(swap, 1, 0, 1, 2);
}

void SwapWindowFrame::createCone(QWidget *parent)
{
	conewidget_ = new QVTKWidget(parent);

	//
	// Next we create an instance of vtkConeSource and set some of its
	// properties. The instance of vtkConeSource "cone" is part of a
	// visualization pipeline (it is a source process object); it produces data
	// (output type is vtkPolyData) which other filters may process.
	//
	vtkConeSource *cone = vtkConeSource::New();
	cone->SetHeight(3.0);
	cone->SetRadius(1.0);
	cone->SetResolution(10);

	//
	// In this example we terminate the pipeline with a mapper process object.
	// (Intermediate filters such as vtkShrinkPolyData could be inserted in
	// between the source and the mapper.)  We create an instance of
	// vtkPolyDataMapper to map the polygonal data into graphics primitives. We
	// connect the output of the cone souece to the input of this mapper.
	//
	vtkPolyDataMapper *coneMapper = vtkPolyDataMapper::New();
	coneMapper->SetInputConnection(cone->GetOutputPort());

	//
	// Create an actor to represent the cone. The actor orchestrates rendering
	// of the mapper's graphics primitives. An actor also refers to properties
	// via a vtkProperty instance, and includes an internal transformation
	// matrix. We set this actor's mapper to be coneMapper which we created
	// above.
	//
	vtkActor *coneActor = vtkActor::New();
	coneActor->SetMapper(coneMapper);

	//
	// Create the Renderer and assign actors to it. A renderer is like a
	// viewport. It is part or all of a window on the screen and it is
	// responsible for drawing the actors it has.  We also set the background
	// color here.
	//
	vtkRenderer *ren1 = vtkRenderer::New();
	ren1->AddActor(coneActor);
	ren1->SetBackground(0.1, 0.2, 0.4);

	//
	// Finally we create the render window which will show up on the screen.
	// We put our renderer into the render window using AddRenderer. We also
	// set the size to be 300 pixels by 300.
	//
	vtkRenderWindow *renWin = conewidget_->GetRenderWindow();
	renWin->AddRenderer(ren1);
	renWin->SetSize(300, 300);

	//
	// Now we loop over 360 degreeees and render the cone each time.
	//
	int i;
	for (i = 0; i < 360; ++i)
	{
		// render the image
		renWin->Render();
		// rotate the active camera by one degree
		ren1->GetActiveCamera()->Azimuth(1);
	}

	//
	// Free up any objects we created. All instances in VTK are deleted by
	// using the Delete() method.
	//
	cone->Delete();
	coneMapper->Delete();
	coneActor->Delete();
	ren1->Delete();
}

void SwapWindowFrame::createSpheres(QWidget *parent)
{
	spherewidget_ = new QVTKWidget(parent);

	// The following lines create a sphere represented by polygons.
	//
	vtkSmartPointer<vtkSphereSource> sphere =
		vtkSmartPointer<vtkSphereSource>::New();
	sphere->SetThetaResolution(100);
	sphere->SetPhiResolution(50);

	// The mapper is responsible for pushing the geometry into the graphics
	// library. It may also do color mapping, if scalars or other attributes
	// are defined.
	//
	vtkSmartPointer<vtkPolyDataMapper> sphereMapper =
		vtkSmartPointer<vtkPolyDataMapper>::New();
	sphereMapper->SetInputConnection(sphere->GetOutputPort());

	// The actor is a grouping mechanism: besides the geometry (mapper), it
	// also has a property, transformation matrix, and/or texture map.
	// In this example we create eight different spheres (two rows of four
	// spheres) and set the specular lighting coefficients. A little ambient
	// is turned on so the sphere is not completely black on the back side.
	//
	vtkSmartPointer<vtkActor> sphere1 =
		vtkSmartPointer<vtkActor>::New();
	sphere1->SetMapper(sphereMapper);
	sphere1->GetProperty()->SetColor(1, 0, 0);
	sphere1->GetProperty()->SetAmbient(0.3);
	sphere1->GetProperty()->SetDiffuse(0.0);
	sphere1->GetProperty()->SetSpecular(1.0);
	sphere1->GetProperty()->SetSpecularPower(5.0);

	vtkSmartPointer<vtkActor> sphere2 =
		vtkSmartPointer<vtkActor>::New();
	sphere2->SetMapper(sphereMapper);
	sphere2->GetProperty()->SetColor(1, 0, 0);
	sphere2->GetProperty()->SetAmbient(0.3);
	sphere2->GetProperty()->SetDiffuse(0.0);
	sphere2->GetProperty()->SetSpecular(1.0);
	sphere2->GetProperty()->SetSpecularPower(10.0);
	sphere2->AddPosition(1.25, 0, 0);

	vtkSmartPointer<vtkActor> sphere3 =
		vtkSmartPointer<vtkActor>::New();
	sphere3->SetMapper(sphereMapper);
	sphere3->GetProperty()->SetColor(1, 0, 0);
	sphere3->GetProperty()->SetAmbient(0.3);
	sphere3->GetProperty()->SetDiffuse(0.0);
	sphere3->GetProperty()->SetSpecular(1.0);
	sphere3->GetProperty()->SetSpecularPower(20.0);
	sphere3->AddPosition(2.5, 0, 0);

	vtkSmartPointer<vtkActor> sphere4 =
		vtkSmartPointer<vtkActor>::New();
	sphere4->SetMapper(sphereMapper);
	sphere4->GetProperty()->SetColor(1, 0, 0);
	sphere4->GetProperty()->SetAmbient(0.3);
	sphere4->GetProperty()->SetDiffuse(0.0);
	sphere4->GetProperty()->SetSpecular(1.0);
	sphere4->GetProperty()->SetSpecularPower(40.0);
	sphere4->AddPosition(3.75, 0, 0);

	vtkSmartPointer<vtkActor> sphere5 =
		vtkSmartPointer<vtkActor>::New();
	sphere5->SetMapper(sphereMapper);
	sphere5->GetProperty()->SetColor(1, 0, 0);
	sphere5->GetProperty()->SetAmbient(0.3);
	sphere5->GetProperty()->SetDiffuse(0.0);
	sphere5->GetProperty()->SetSpecular(0.5);
	sphere5->GetProperty()->SetSpecularPower(5.0);
	sphere5->AddPosition(0.0, 1.25, 0);

	vtkSmartPointer<vtkActor> sphere6 =
		vtkSmartPointer<vtkActor>::New();
	sphere6->SetMapper(sphereMapper);
	sphere6->GetProperty()->SetColor(1, 0, 0);
	sphere6->GetProperty()->SetAmbient(0.3);
	sphere6->GetProperty()->SetDiffuse(0.0);
	sphere6->GetProperty()->SetSpecular(0.5);
	sphere6->GetProperty()->SetSpecularPower(10.0);
	sphere6->AddPosition(1.25, 1.25, 0);

	vtkSmartPointer<vtkActor> sphere7 =
		vtkSmartPointer<vtkActor>::New();
	sphere7->SetMapper(sphereMapper);
	sphere7->GetProperty()->SetColor(1, 0, 0);
	sphere7->GetProperty()->SetAmbient(0.3);
	sphere7->GetProperty()->SetDiffuse(0.0);
	sphere7->GetProperty()->SetSpecular(0.5);
	sphere7->GetProperty()->SetSpecularPower(20.0);
	sphere7->AddPosition(2.5, 1.25, 0);

	vtkSmartPointer<vtkActor> sphere8 =
		vtkSmartPointer<vtkActor>::New();
	sphere8->SetMapper(sphereMapper);
	sphere8->GetProperty()->SetColor(1, 0, 0);
	sphere8->GetProperty()->SetAmbient(0.3);
	sphere8->GetProperty()->SetDiffuse(0.0);
	sphere8->GetProperty()->SetSpecular(0.5);
	sphere8->GetProperty()->SetSpecularPower(40.0);
	sphere8->AddPosition(3.75, 1.25, 0);

	// Create the graphics structure. The renderer renders into the
	// render window. The render window interactor captures mouse events
	// and will perform appropriate camera or actor manipulation
	// depending on the nature of the events.
	//
	vtkSmartPointer<vtkRenderer> ren1 =
		vtkSmartPointer<vtkRenderer>::New();

	vtkRenderWindow *renWin = spherewidget_->GetRenderWindow();

	renWin->AddRenderer(ren1);

	// Add the actors to the renderer, set the background and size.
	//
	ren1->AddActor(sphere1);
	ren1->AddActor(sphere2);
	ren1->AddActor(sphere3);
	ren1->AddActor(sphere4);
	ren1->AddActor(sphere5);
	ren1->AddActor(sphere6);
	ren1->AddActor(sphere7);
	ren1->AddActor(sphere8);
	ren1->SetBackground(0.1, 0.2, 0.4);
	renWin->SetSize(400, 200);

	// Set up the lighting.
	//
	vtkSmartPointer<vtkLight> light =
		vtkSmartPointer<vtkLight>::New();
	light->SetFocalPoint(1.875, 0.6125, 0);
	light->SetPosition(0.875, 1.6125, 1);
	ren1->AddLight(light);

	// We want to eliminate perspective effects on the apparent lighting.
	// Parallel camera projection will be used. To zoom in parallel projection
	// mode, the ParallelScale is set.
	//
	ren1->GetActiveCamera()->SetFocalPoint(0, 0, 0);
	ren1->GetActiveCamera()->SetPosition(0, 0, 1);
	ren1->GetActiveCamera()->SetViewUp(0, 1, 0);
	ren1->GetActiveCamera()->ParallelProjectionOn();
	ren1->ResetCamera();
	ren1->GetActiveCamera()->SetParallelScale(1.5);

}

void SwapWindowFrame::swap()
{
	QWidget *rightWidget = rightLayout_->itemAt(0)->widget();
	QWidget *leftWidget = leftLayout_->itemAt(0)->widget();
	rightLayout_->removeWidget(rightWidget);
	leftLayout_->removeWidget(leftWidget);

	rightWidget->setParent(leftPanel_);
	leftWidget->setParent(rightPanel_);

	leftLayout_->addWidget(rightWidget);
	rightLayout_->addWidget(leftWidget);
}